/**
 * @author [Tristan Valcke]{@link https://github.com/Itee}
 * @license [MIT]{@link https://opensource.org/licenses/MIT}
 *
 * @file Todo
 *
 * @example Todo
 *
 */


const TMongoDBPlugin            = require( 'itee-database' ).TMongoDBPlugin
const TAbstractConverterManager = require( 'itee-database' ).TAbstractConverterManager

const JsonToThree    = require( './converters/JsonToThree' )
const ShpToThree     = require( './converters/ShpToThree' )
const DbfToThree     = require( './converters/DbfToThree' )
const MtlToThree     = require( './converters/MtlToThree' )
const ObjToThree     = require( './converters/Obj2ToThree' )
const ThreeToMongoDB = require( './converters/ThreeToMongoDB' )

let instance = null
if ( !instance ) {

    instance = new TMongoDBPlugin()
        .addDescriptor( {
            route:      '/objects',
            controller: {
                options: {
                    schemaName: 'Objects3D'
                },
                can:     {
                    create: {
                        on:   'put',
                        over: '/(:id)?'
                    },
                    read:   {
                        on:   'post',
                        over: '/(:id)?'
                    },
                    update: {
                        on:   'patch',
                        over: '/(:id)?'
                    },
                    delete: {
                        on:   'delete',
                        over: '/(:id)?'
                    }
                }
            }
        } )
        .addDescriptor( {
            route:      '/curves',
            controller: {
                options: {
                    schemaName: 'Curves'
                },
                can:     {
                    create: {
                        on:   'put',
                        over: '/(:id)?'
                    },
                    read:   {
                        on:   'post',
                        over: '/(:id)?'
                    },
                    update: {
                        on:   'patch',
                        over: '/(:id)?'
                    },
                    delete: {
                        on:   'delete',
                        over: '/(:id)?'
                    }
                }
            }
        } )
        .addDescriptor( {
            route:      '/geometries',
            controller: {
                options: {
                    schemaName: 'Geometries'
                },
                can:     {
                    create: {
                        on:   'put',
                        over: '/(:id)?'
                    },
                    read:   {
                        on:   'post',
                        over: '/(:id)?'
                    },
                    update: {
                        on:   'patch',
                        over: '/(:id)?'
                    },
                    delete: {
                        on:   'delete',
                        over: '/(:id)?'
                    }
                }
            }
        } )
        .addDescriptor( {
            route:      '/materials',
            controller: {
                options: {
                    schemaName: 'Materials'
                },
                can:     {
                    create: {
                        on:   'put',
                        over: '/(:id)?'
                    },
                    read:   {
                        on:   'post',
                        over: '/(:id)?'
                    },
                    update: {
                        on:   'patch',
                        over: '/(:id)?'
                    },
                    delete: {
                        on:   'delete',
                        over: '/(:id)?'
                    }
                }
            }
        } )
        .addDescriptor( {
            route:      '/textures',
            controller: {
                options: {
                    schemaName: 'Textures'
                },
                can:     {
                    create: {
                        on:   'put',
                        over: '/(:id)?'
                    },
                    read:   {
                        on:   'post',
                        over: '/(:id)?'
                    },
                    update: {
                        on:   'patch',
                        over: '/(:id)?'
                    },
                    delete: {
                        on:   'delete',
                        over: '/(:id)?'
                    }
                }
            }
        } )

        .addController( TAbstractConverterManager )
        .addDescriptor( {
            route:      '/uploads',
            controller: {
                name:    'TAbstractConverterManager',
                options: {
                    useNext:    true,
                    converters: {
                        JsonToThree: new JsonToThree(),
                        ShpToThree:  new ShpToThree(),
                        DbfToThree:  new DbfToThree(),
                        MtlToThree:  new MtlToThree(),
                        ObjToThree:  new ObjToThree()
                    },
                    rules:      [
                        {
                            on:  '.json',
                            use: 'JsonToThree'
                        },
                        {
                            on:  '.shp',
                            use: 'ShpToThree'
                        },
                        {
                            on:  '.dbf',
                            use: 'DbfToThree'
                        },
                        {
                            on:  [ '.shp', '.dbf' ],
                            use: [ 'ShpToThree', 'DbfToThree' ]
                        },
                        {
                            on:  '.mtl',
                            use: 'MtlToThree'
                        },
                        {
                            on:  '.obj',
                            use: 'ObjToThree'
                        },
                        {
                            on:  [ '.mtl', '.obj' ],
                            use: [ 'MtlToThree', 'ObjToThree' ]
                        }
                    ],
                    inserter:   ThreeToMongoDB
                },
                can:     {
                    processFiles: {
                        on:   'post',
                        over: '/'
                    }
                }
            }
        } )

}

module.exports = instance
