/**
 * @author [Tristan Valcke]{@link https://github.com/Itee}
 * @license [BSD-3-Clause]{@link https://opensource.org/licenses/BSD-3-Clause}
 *
 * @module Schemas/Geometry
 *
 * @description Todo...
 */

let _schema = undefined

function getSchemaFrom ( Mongoose ) {
    'use strict'

    if ( !_schema ) {
        _createSchema( Mongoose )
    }

    return _schema

}

function _createSchema ( Mongoose ) {
    'use strict'

    const Schema = Mongoose.Schema
    const Types  = Schema.Types
    const Mixed  = Types.Mixed

    const ONE_BYTE    = 1
    const TWO_BYTE    = 2
    const FOUR_BYTE   = 4
    const HEIGHT_BYTE = 8

    const ArrayType = {
        Int8Array:         0,
        Uint8Array:        1,
        Uint8ClampedArray: 2,
        Int16Array:        3,
        Uint16Array:       4,
        Int32Array:        5,
        Uint32Array:       6,
        Float32Array:      7,
        Float64Array:      8
    }

    _schema = new Schema( {
        array:       {
            type: Buffer,
            set:  ( array ) => {

                const arrayLength      = array.length
                const arrayConstructor = array.constructor

                let buffer = null
                let offset = 0

                if ( arrayConstructor === Int8Array ) {

                    buffer = Buffer.allocUnsafe( ONE_BYTE + arrayLength * ONE_BYTE )
                    offset = buffer.writeUInt8( ArrayType.Int8Array, offset )

                    for ( let index = 0 ; index < arrayLength ; index++ ) {
                        offset = buffer.writeInt8( array[ index ], offset )
                    }

                } else if ( arrayConstructor === Uint8Array ) {

                    buffer = Buffer.allocUnsafe( ONE_BYTE + arrayLength * ONE_BYTE )
                    offset = buffer.writeUInt8( ArrayType.Uint8Array, offset )

                    for ( let index = 0 ; index < arrayLength ; index++ ) {
                        offset = buffer.writeUInt8( array[ index ], offset )
                    }

                } else if ( arrayConstructor === Uint8ClampedArray ) {

                    buffer = Buffer.allocUnsafe( ONE_BYTE + arrayLength * ONE_BYTE )
                    offset = buffer.writeUInt8( ArrayType.Uint8ClampedArray, offset )

                    for ( let index = 0 ; index < arrayLength ; index++ ) {
                        offset = buffer.writeUInt8( array[ index ], offset )
                    }

                } else if ( arrayConstructor === Int16Array ) {

                    buffer = Buffer.allocUnsafe( ONE_BYTE + arrayLength * TWO_BYTE )
                    offset = buffer.writeUInt8( ArrayType.Int16Array, offset )

                    for ( let index = 0 ; index < arrayLength ; index++ ) {
                        offset = buffer.writeInt16BE( array[ index ], offset )
                    }

                } else if ( arrayConstructor === Uint16Array ) {

                    buffer = Buffer.allocUnsafe( ONE_BYTE + arrayLength * TWO_BYTE )
                    offset = buffer.writeUInt8( ArrayType.Uint16Array, offset )

                    for ( let index = 0 ; index < arrayLength ; index++ ) {
                        offset = buffer.writeUInt16BE( array[ index ], offset )
                    }

                } else if ( arrayConstructor === Int32Array ) {

                    buffer = Buffer.allocUnsafe( ONE_BYTE + arrayLength * FOUR_BYTE )
                    offset = buffer.writeUInt8( ArrayType.Int32Array, offset )

                    for ( let index = 0 ; index < arrayLength ; index++ ) {
                        offset = buffer.writeInt32BE( array[ index ], offset )
                    }

                } else if ( arrayConstructor === Uint32Array ) {

                    buffer = Buffer.allocUnsafe( ONE_BYTE + arrayLength * FOUR_BYTE )
                    offset = buffer.writeUInt8( ArrayType.Uint32Array, offset )

                    for ( let index = 0 ; index < arrayLength ; index++ ) {
                        offset = buffer.writeUInt32BE( array[ index ], offset )
                    }

                } else if ( arrayConstructor === Float32Array ) {

                    buffer = Buffer.allocUnsafe( ONE_BYTE + arrayLength * FOUR_BYTE )
                    offset = buffer.writeUInt8( ArrayType.Float32Array, offset )

                    for ( let index = 0 ; index < arrayLength ; index++ ) {
                        offset = buffer.writeFloatBE( array[ index ], offset )
                    }

                } else if ( arrayConstructor === Float64Array ) {

                    buffer = Buffer.allocUnsafe( ONE_BYTE + arrayLength * HEIGHT_BYTE )
                    offset = buffer.writeUInt8( ArrayType.Float64Array, offset )

                    for ( let index = 0 ; index < arrayLength ; index++ ) {
                        offset = buffer.writeDoubleBE( array[ index ], offset )
                    }

                } else {

                    throw new Error( 'Unable to determine the array type to bufferize.' )

                }

                return buffer

            }
        },
        count:       Number,
        dynamic:     Boolean,
        itemSize:    Number,
        name:        String,
        needsUpdate: Boolean,
        normalized:  Boolean,
        updateRange: Mixed,
        uuid:        String,
        version:     Number
    }, {
        _id: false,
        id:  false
    } )

}

module.exports = {
    getSchemaFrom:   getSchemaFrom,
    getModelFrom:    Mongoose => null,
    registerModelTo: Mongoose => Mongoose
}
